#ifndef _I386_FUNCTIONS_H
#define _I386_FUNCTIONS_H

/*
 *	Functions for analysing i386 architecture systems
*/

#include "../../globals.h"

#include "magic.h" 

/*
 * Function that manages mapping from linear to physical address space.
 * 
 */
int i386_lin_read_linear(unsigned long PGD, unsigned long lin_addr, void *out_data, unsigned int priv_block_size, unsigned long *dump_offset );

/*
 * Function for verifying if given address contains structure list_head.
 *
 * Verification is done by finding and checking double linked 
 * "list_head". This way is chosen instead of classic 
 * pattern matching, because I believe it is more flexible and 
 * can be used for almost every 2.6 kernel version. (even future;)
 * It does not depend on task_struct definition too much, only 
 * important characteristic is number of list_heads between 
 * "comm" array and list_head "tasks". This is specified by
 * TS_MAGIC_HEADLIST_NUMBER. 
 *
 * Correct list_head can be verified easily:
 * Data at address point to (virtual) address where is stored another
 * list_head. Second pointer in this second head list point back
 * to (virtual) address where we started.
 *
 *  ...   |--------> |---------->  |---------->   ...
 *          <------  | <-----------| <-------- 
 *
 * Note: This will not find processes, that are inactive,
 * has already ended in past, or something similiar. Therefore,
 * if you want to mine as much data as possible, and don't 
 * afraid of too much false positives, use different method.
 * (idetect toolkit, or some greping shell utilities)
 *
 *
 *  verify_list_head (address to verify )
 *  return 0 on success, otherwise error > 0
 *
 * */

int i386_lin_verify_list_head(unsigned long ver_addr);


/*
 *  verify_vfsmount (pointer to vfsmount structure)
 *  return 0 on success, otherwise -1
 *
 *  vfsmount structure is "recognized" by simple test:
 *  should contain pointer to string "/dev/".
 *  (char *mnt_name)
 *  This is alone not a strong heuristic, but in combination
 *  with specific chain where structure should be found when trying 
 *  to list open files, it is enough.
 */

int i386_lin_verify_vfsmount(unsigned long ver_addr);

/*
 *  verify_qstr (pointer to "qstr" structure )
 *  return 0 on success, otherwise error
 *
 *  Used for listing of open files per process.
 *    d_name -> name
 *    (qstr)   (char)
 *
 *  This function verifies, if pointer points to qstr structure.
 *  qstr is elemental structure, and we assume, that will not change (there is no reason)
 *  Verification is done by checking, if string that "name" points to is really "len"
 *  bytes long (looking for \0) and "len" is not 0.
 *
 *  This is qstr definition:
 *
 *  struct qstr {
 *          unsigned int hash;
 *          unsigned int len;
 *          const unsigned char *name;
 *  };
 *                          
 *
 */

int i386_lin_verify_qstr(unsigned long ver_addr);
/*
 *  verify_dentry (pointer to "dentry" structure )
 *  return 0 on success, otherwise -1
 *  determined distances are saved into image1
 *
 *  Used for listing of open files per process.
 *  Chain we are following looks like:
 *
 *  => task_struct ->       files   ->      fdt     ->      fd      ->      f_path  ->      dentry  ->      d_name-> name
 *  		    (files_struct)      (fdtable)        (file **)          (path)          (dentry)        (qstr)  (char)
 *
 *  This function determines distances in structure dentry
 *  It is better to determine structures by more "sub" heuristics, because amount of tries needed is decreased.
 *  (and amount of false positives too)
 *  
 */

int i386_lin_verify_dentry(unsigned long ver_addr); 

/*
 *  determine_up_to_path (pointer to "tasks" list_head in task_struct )
 *  return 0 on success, otherwise -1
 *  determined distances are saved into image1
 *
 *  function that tries to determine distances in structures needed
 *  for listing of open files per process up to f_path structure.
 *  Chain we are following looks like:
 *
 *  => task_struct ->       files   ->      fdt     ->      fd      ->      f_path  ->      dentry  ->      d_name-> name
 *  		    (files_struct)      (fdtable)        (file **)          (path)          (dentry)        (qstr)  (char)
 *
 *  So this function determines distances in structures files_struct, fdtable, file and path.
 *  It is better to determine structures by more "sub" heuristics, because amount of tries needed is decreased.
 *  (and amount of false positives too)
 *  
 */

int i386_lin_determine_up_to_path(unsigned long ver_addr);

/*
 *  determine_open_files (pointer to "tasks" list_head in task_struct )
 *  return 0 on success, otherwise -1
 *
 *  function that tries to determine distances in all structures needed
 *  for listing of open files per process. 
 *  This will definitely not work in 2.4 kernels.
 */

int i386_lin_determine_open_files(unsigned long ver_addr);

/*
 *  verify_task_struct (pointer to "comm" field of task_struct )
 *  return 0 on success, otherwise -1
 */

int i386_lin_verify_task_struct(unsigned long ver_addr);

/*
 *
 *  Interesting fields in "module" are just near. This should not change (no reason),
 *  so this function is much simpler then verify_task_struct .
 *
 *  verify_module (pointer to "name" field of module )
 *  return distance between name and list_head in bytes on success, otherwise < 0
 *
 *  
 */

int i386_lin_verify_module(unsigned long ver_addr);

/*
 * Function for verifying if given address contains VPT 
 * (resp. PaGeDirectory) of some process.
 *
 * Verification is done by simple checking
 * if flags for first three records beyond 0xC0000000
 * (beginning of kernel address space) are same.
 * This should usually be true, as userspace PGD are 
 * cloned from kernel PGD, but it is just heuristic, 
 * and can be wrong.
 *
 *  user_VPT_test (linear address to verify )
 *  return 0 on success, otherwise error > 0
 *
 * */

int i386_lin_user_VPT_test(unsigned long my_addr);

/*
 * Function for verifying if given address contains system VPT 
 * (resp. PaGeDirectory)
 *
 * Verification is done by simple checking
 * if first VPT_MAGIC_ZEROES bytes contain only zeroes and 
 * if flags for first three records beyond 0xC0000000
 * (beginning of kernel address space) are same.
 * This should usually be true, but it is just heuristic, 
 * and can be wrong.
 *
 *  VPT_test (physical address to verify )
 *  return 0 on success, otherwise error > 0
 *
 * */

int i386_lin_VPT_test(int my_addr);

/*
 * Function for finding VPT of given process 
 * (resp. PaGeDirectory).
 *
 * This function is assumed to be called only once.
 * When distances used in analyzed image are known,
 * it is faster and safer to use them directly.
 *
 * Function tries address around values that was used in 
 * available linux 2.6 kernel, and tries, if they match definition.
 * Success is not guaranteed, but it was working nice even for kernels
 * with random debugging symbols compiled in.
 *
 * find_process_VPT (linear address of "tasks" list_head)
 * return 0 on success, otherwise error > 0
 * Discovered distances are saved in glibal structure image1.
 *
 * */

int i386_lin_find_process_VPT(unsigned long my_addr);

/*
 * Function that tries to determine general information 
 * about analyzed image.
 *
 * for now, only system VPT is found.
 *
 * determine ()
 * return 0 on success, otherwise error > 0
 * Discovered information are saved in global structure image1.
 *
 * */

int i386_lin_determine();

/* Only printf processes for now.
 * listhead is address of list_head of one of the processes
 * distance is in bytes from given listhead structure to comm (char[16]) field
 */

int i386_lin_list_processes(unsigned long listhead,int distance);

/*
	We do not search for in init directly, because it is too common string in memory => takes too long
	This is almost clone of list_processes
*/

int i386_lin_find_userspace_process();

/* Another clone of list_processes
 * Only number of members of list_head
 *
 * input: Physical address of list_head
 * output: Number of members, -3 if infinite loop detected
 */

int i386_lin_list_head_size(unsigned long listhead);

/* Only printf modules for now.
 * listhead is address of list_head of one of the modules (not linear address, but address in physical dump file)
 * distance is in bytes from given listhead structure to comm (char[16]) field
 */


int i386_lin_list_modules(unsigned long listhead,int distance);

/*
 * Function that tries to determine linux-specific information 
 * about analyzed image.
 *
 * Some process and then some userland process are found,
 * and distance between important fields are determined.
 *
 * Information about "module" structure are determined separately,
 * because module support is only optional.
 *
 * linux_determine ()
 * return 0 on success, otherwise error <> 0
 * Discovered information are saved in global structure image1.
 *
 * */

int i386_lin_linux_determine();

/*
 * Function that tries to determine linux module-specific 
 * information about analyzed image.
 *
 * Some module is found,
 * and distance between important fields are determined.
 *
 * This is done separately,
 * because module support is only optional, and is not 
 * critical for another analyze.
 *
 * linux_determine_module ()
 * return 0 on success, otherwise error <> 0
 * Discovered information are saved in global structure image1.
 *
 * */

int i386_lin_linux_determine_module();

#endif
